#pragma once

namespace deepmd {

template <typename FPTYPE>
void prod_force_a_cpu(FPTYPE* force,
                      const FPTYPE* net_deriv,
                      const FPTYPE* in_deriv,
                      const int* nlist,
                      const int nloc,
                      const int nall,
                      const int nnei,
                      const int start_index = 0);

template <typename FPTYPE>
void prod_force_r_cpu(FPTYPE* force,
                      const FPTYPE* net_deriv,
                      const FPTYPE* in_deriv,
                      const int* nlist,
                      const int nloc,
                      const int nall,
                      const int nnei);

#if GOOGLE_CUDA
template <typename FPTYPE>
void prod_force_a_gpu_cuda(FPTYPE* force,
                           const FPTYPE* net_deriv,
                           const FPTYPE* in_deriv,
                           const int* nlist,
                           const int nloc,
                           const int nall,
                           const int nnei);

template <typename FPTYPE>
void prod_force_r_gpu_cuda(FPTYPE* force,
                           const FPTYPE* net_deriv,
                           const FPTYPE* in_deriv,
                           const int* nlist,
                           const int nloc,
                           const int nall,
                           const int nnei);
#endif  // GOOGLE_CUDA

#if TENSORFLOW_USE_ROCM
template <typename FPTYPE>
void prod_force_a_gpu_rocm(FPTYPE* force,
                           const FPTYPE* net_deriv,
                           const FPTYPE* in_deriv,
                           const int* nlist,
                           const int nloc,
                           const int nall,
                           const int nnei);

template <typename FPTYPE>
void prod_force_r_gpu_rocm(FPTYPE* force,
                           const FPTYPE* net_deriv,
                           const FPTYPE* in_deriv,
                           const int* nlist,
                           const int nloc,
                           const int nall,
                           const int nnei);
#endif  // TENSORFLOW_USE_ROCM

}  // namespace deepmd
